import json
import logging
import os
import subprocess
from pathlib import Path

from . import config_util
from .util import PROCESS_DEFAULT, PROFILE_DEFAULT, generate_cmd, generate_process_options

logger = logging.getLogger()

KTRL_PROFILE = PROFILE_DEFAULT
KTRL_INSTANCE = {
    "package": "",
    "version": "",
    "file": "",
    "dbPath": "",
    "args": [],
}
KTRL_PROCESS = KTRL_INSTANCE.copy()
KTRL_PROCESS.update(PROCESS_DEFAULT)


global_profile_dir = Path.joinpath(config_util.global_kuki_root, "_profile")
global_process_dir = Path.joinpath(config_util.global_kuki_root, "_process")
local_profile_dir = Path.joinpath(Path("ktrl"), "profile")
local_process_dir = Path.joinpath(Path("ktrl"), "process")


def dump_json(path: Path, config):
    with open(path, "w") as file:
        file.write(config)


def config_file(name: str, type: str, globalMode: False):
    if type == "profile":
        path = Path.joinpath(
            global_profile_dir if globalMode else local_profile_dir,
            name + ".profile.json",
        )
        config = KTRL_PROFILE
        all_keys = set(KTRL_PROFILE.keys())
    else:
        path = Path.joinpath(
            global_process_dir if globalMode else local_process_dir,
            name + ".process.json",
        )
        config = KTRL_INSTANCE.copy()
        all_keys = set(KTRL_PROCESS.keys())
        if not globalMode:
            config.pop("package")
            config.pop("version")

    default_keys = set(config.keys())
    path.parent.mkdir(parents=True, exist_ok=True)
    if path.exists():
        config.update(json.loads(path.read_text()))
        for key in config.keys():
            if key not in all_keys:
                logger.warn("unrecognized key - {}".format(key))
                delattr(config, key)
        keys = config.keys()
    else:
        keys = default_keys

    for key in sorted(keys):
        current_value = config.get(key, "")
        input_value = input("{} ({}): ".format(key, current_value)).strip()
        if key == "args":
            input_value = input_value.split()
        config[key] = input_value if input_value else current_value
        if key in ["file", "package", "version"] and not config[key]:
            logger.error(key + " cannot be empty string")
            return

    config_json = json.dumps(config, indent=2)
    logger.info("About to write to {}".format(path))
    logger.info("\n" + config_json)
    proceed = input("Is this OK? (YES/no) ").strip()
    if not proceed or proceed.lower() == "yes":
        dump_json(path, config_json)


def list_config(type: str, globalMode: False):
    if type == "profile":
        path = global_profile_dir if globalMode else local_profile_dir
    else:
        path = global_process_dir if globalMode else local_process_dir

    for file in path.glob("*.json"):
        print(file)


def start(profile_name: str, process_name: str, globalMode: False):
    profile_path = Path.joinpath(
        global_profile_dir if globalMode else local_profile_dir,
        profile_name + ".profile.json",
    )
    process_path = Path.joinpath(
        global_process_dir if globalMode else local_process_dir,
        process_name + ".process.json",
    )
    if profile_path.exists():
        profile_json: dict = json.loads(profile_path.read_text())
    else:
        logger.error("Not such file - {}".format(profile_path))
        return

    if process_path.exists():
        process_json: dict = json.loads(process_path.read_text())
    else:
        logger.error("Not such file - {}".format(process_path))
        return

    if globalMode:
        package_path = Path.joinpath(
            config_util.global_kuki_root,
            process_json.get("package"),
            process_json.get("version"),
        )
        if not package_path.exists():
            logger.error("Not such folder - {}".format(package_path))
            return
        os.chdir(package_path)
    else:
        package_path = Path.cwd()

    file_name: str = process_json.get("file")

    file_path = Path.joinpath(
        package_path,
        "src",
        file_name[4:] if file_name.startswith("src/") else file_name,
    )

    options = generate_process_options([], process_json)
    # generate run command
    options = (
        ["-kScriptType", "ktrl"]
        + ["-file", str(file_path)]
        + ["-dbPath", process_json.get("dbPath", [])]
        + process_json.get("args")
        + options
    )
    cmd = generate_cmd(options, profile_json)

    logger.info("starting " + cmd)

    try:
        subprocess.run(cmd, shell=True, check=True)
    except subprocess.CalledProcessError:
        exit(1)



import json
import logging
import os
from pathlib import Path
from typing import TypedDict

logger = logging.getLogger()

global_kuki_root = (
    Path(os.getenv("KUKIPATH")) if os.getenv("KUKIPATH") else Path.joinpath(Path.home(), "kuki")
)

global_config_dir = Path.joinpath(global_kuki_root, "_config")

global_config_dir.mkdir(parents=True, exist_ok=True)

config_file = "kukirc.json"

global_config_path = Path.joinpath(global_config_dir, config_file)


class Kukirc(TypedDict):
    registry: str
    token: str


def load_config() -> Kukirc:
    if not Path.exists(global_config_path):
        with open(global_config_path, "w") as file:
            file.write(json.dumps({}))
        return {}
    else:
        with open(global_config_path, "r") as file:
            return json.load(file)


def update_config(field: str, value: str):
    if not value:
        kukirc = load_config()
        delattr(kukirc, field)
        logger.info("Empty value for {}, removing existing value".format(field))
    else:
        logger.info("update '{}' of {}".format(field, config_file))
        kukirc = load_config()
        kukirc[field] = value
    dump_config(kukirc)


def dump_config(config: Kukirc):
    logger.info("persist update to {}".format(config_file))
    with open(global_config_path, "w") as file:
        file.write(json.dumps(config, indent=2))



import json
import logging
import os
import re
from pathlib import Path
from typing import Dict, List, TypedDict

logger = logging.getLogger()
config_file = "kuki.json"
index_file = "kuki_index.json"
readme_file = "README.md"

package_path = Path.cwd()
readme_path = Path(readme_file)
src_path = Path("src")
package_config_path = Path(config_file)
package_include_path = Path(".kukiinclude")
package_index_path = Path(index_file)


class Kuki(TypedDict):
    name: str
    version: str
    description: str
    author: str
    git: str
    type: str
    dependencies: Dict[str, str]


def generate_json(name: str, description="", author="", git="", package_type="q"):
    if package_config_path.exists():
        overwrite = input("kuki.json already exists, overwrite: (yes/No) ").strip()
        if not overwrite or not overwrite.lower() in ["yes"]:
            return
    kuki: Kuki = {
        "name": name,
        "version": "0.0.1",
        "description": description,
        "author": author,
        "git": git,
        "type": package_type,
        "dependencies": {},
    }
    kuki_json = json.dumps(kuki, indent=2)
    logger.info("About to write to {}".format(package_config_path))
    logger.info("\n" + kuki_json)
    proceed = input("Is this OK? (YES/no) ").strip()
    if not proceed or proceed.lower() == "yes":
        dump_kuki(kuki)
        readme_path.touch()
        readme_path.write_text("# {name}\n\n- author: {author}\n- git: {git}\n".format(**kuki))
        package_index_path.write_text("{}")
        src_path.mkdir(parents=True, exist_ok=True)


def init():
    dir = os.path.basename(os.getcwd())
    package = input("package name: ({}) ".format(dir.lower())).strip()
    if not package:
        package = dir.lower()

    is_valid_name(package)

    description = input("description: ").strip()
    author = input("author: ").strip()
    git = input("git repository: ").strip()
    package_type = input("package type: (q)").strip()
    if not package_type:
        package_type = "q"
    if package_type not in ["q", "k", "k9"]:
        logger.error("only support q, k, or k9 package type")
        return

    generate_json(package, description, author, git, package_type)


def dump_kuki(kuki: Kuki):
    with open(package_config_path, "w") as file:
        file.write(json.dumps(kuki, indent=2))


def exits():
    return package_config_path.exists()


def roll_up_version(type: str):
    kuki: Kuki = json.loads(package_config_path.read_text())
    logger.info("roll up version")
    logger.info("from - " + kuki["version"])
    [major, minor, patch] = list(map(int, kuki["version"].split(".")))
    if type == "patch":
        patch += 1
    elif type == "minor":
        minor += 1
        patch = 0
    elif type == "major":
        major += 1
        minor = 0
        patch = 0
    version = "{}.{}.{}".format(major, minor, patch)
    kuki["version"] = version
    logger.info("to   - " + version)
    dump_kuki(kuki)


def load_kuki() -> Kuki:
    if package_config_path.exists():
        return json.loads(package_config_path.read_text())
    else:
        return {}


def load_include() -> List[str]:
    includes = set(["src/*", "cfg/*", config_file, index_file, readme_file])
    if package_include_path.exists():
        with open(package_include_path, "r") as file:
            while line := file.readline():
                if line.strip() != "":
                    includes.add(line.strip())
    return includes


def load_readme() -> str:
    with open(readme_path, "r") as file:
        return file.read()


def load_pkg_index() -> Dict[str, Kuki]:
    if package_index_path.exists():
        with open(package_index_path, "r") as file:
            return json.load(file)
    else:
        return {}


def dump_pkg_index(kuki_index: Dict[str, Kuki]):
    with open(package_index_path, "w") as file:
        json.dump(kuki_index, file, indent=2)


def is_valid_name(name: str) -> bool:
    if re.fullmatch(r"(@[a-z-]+/)?[a-z-]+", name) is None:
        logger.error("only allows lower cases(a-z) and hyphen(-) as the package name")
        exit(1)
